@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: client_channel.c,v 1.37 2002/06/19 12:04:03 priikone Exp $ */
+/* $Id: client_channel.c,v 1.39 2002/10/14 19:04:36 priikone Exp $ */
 /* This file includes channel message sending and receiving routines,
    channel key receiving and setting, and channel private key handling 
    routines. */
@@ -114,8 +114,7 @@ void silc_client_send_channel_message(SilcClient client,
     for (i = 0; i < iv_len; i++) channel->iv[i] = 
 				   silc_rng_get_byte(client->rng);
   else
-    silc_hash_make(client->internal->md5hash, channel->iv, iv_len, 
-		   channel->iv);
+    silc_hash_make(client->md5hash, channel->iv, iv_len, channel->iv);
 
   /* Encode the channel payload. This also encrypts the message payload. */
   payload = silc_channel_message_payload_encode(flags, data_len, data, iv_len, 
@@ -308,7 +307,7 @@ void silc_client_channel_message(SilcClient client,
     SilcChannelClientResolve res = silc_calloc(1, sizeof(*res));
     res->payload = payload;
     res->channel_id = id;
-    silc_client_get_client_by_id_resolve(client, conn, client_id,
+    silc_client_get_client_by_id_resolve(client, conn, client_id, NULL,
 					 silc_client_channel_message_cb,
 					 res);
     payload = NULL;
@@ -513,7 +512,7 @@ int silc_client_add_channel_private_key(SilcClient client,
   /* Produce the key material */
   keymat = silc_calloc(1, sizeof(*keymat));
   if (silc_ske_process_key_material_data(key, key_len, 16, 256, 16, 
-					 client->internal->md5hash, keymat) 
+					 client->md5hash, keymat) 
       != SILC_SKE_STATUS_OK)
     return FALSE;
 
