@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: idlist.c,v 1.54 2002/06/22 20:53:16 priikone Exp $ */
+/* $Id: idlist.c,v 1.57 2002/10/11 14:24:53 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcclient.h"
@@ -158,6 +158,7 @@ void silc_client_get_clients(SilcClient client,
 			     void *context)
 {
   GetClientInternal i;
+  int len;
   char *userhost;
 
   if (!nickname)
@@ -172,13 +173,13 @@ void silc_client_get_clients(SilcClient client,
   i->context = context;
 
   if (nickname && server) {
-    userhost = silc_calloc(strlen(nickname) + strlen(server) + 2,
-			   sizeof(*userhost));
-    strncat(userhost, nickname, strlen(nickname));
-    strncat(userhost, "@", 1);
-    strncat(userhost, server, strlen(server));
+    len = strlen(nickname) + strlen(server) + 3;
+    userhost = silc_calloc(len, sizeof(*userhost));
+    silc_strncat(userhost, len, nickname, strlen(nickname));
+    silc_strncat(userhost, len, "@", 1);
+    silc_strncat(userhost, len, server, strlen(server));
   } else {
-    userhost = strdup(nickname);
+    userhost = silc_memdup(nickname, strlen(nickname));
   }
 
   /* Register our own command reply for this command */
@@ -547,6 +548,7 @@ SILC_CLIENT_CMD_FUNC(get_client_by_id_callback)
 void silc_client_get_client_by_id_resolve(SilcClient client,
 					  SilcClientConnection conn,
 					  SilcClientID *client_id,
+					  SilcBuffer attributes,
 					  SilcGetClientCallback completion,
 					  void *context)
 {
@@ -569,7 +571,9 @@ void silc_client_get_client_by_id_resolve(SilcClient client,
   /* Send the command */
   idp = silc_id_payload_encode(client_id, SILC_ID_CLIENT);
   silc_client_command_send(client, conn, SILC_COMMAND_WHOIS, conn->cmd_ident,
-			   1, 4, idp->data, idp->len);
+			   2, 3, attributes ? attributes->data : NULL,
+			   attributes ? attributes->len : 0,
+			   4, idp->data, idp->len);
   silc_buffer_free(idp);
 
   /* Add pending callback */
@@ -646,9 +650,14 @@ void silc_client_update_client(SilcClient client,
 
   SILC_LOG_DEBUG(("Start"));
 
-  if (!client_entry->username && username)
-    silc_parse_userfqdn(username, &client_entry->username, 
+  if ((!client_entry->username || !client_entry->hostname) && username) {
+    silc_free(client_entry->username);
+    silc_free(client_entry->hostname);
+    client_entry->username = NULL;
+    client_entry->hostname = NULL;
+    silc_parse_userfqdn(username, &client_entry->username,
 			&client_entry->hostname);
+  }
   if (!client_entry->realname && userinfo)
     client_entry->realname = strdup(userinfo);
   if (!client_entry->nickname && nickname) {
