@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silcconfig.c,v 1.10 2002/04/08 18:03:37 johnny Exp $ */
+/* $Id: silcconfig.c,v 1.14 2002/10/11 14:20:33 priikone Exp $ */
 
 #include "silcincludes.h"
 
@@ -273,14 +273,6 @@ SilcConfigFile *silc_config_open(const char *configfile)
 void silc_config_close(SilcConfigFile *file)
 {
   if (file) {
-    /* XXX FIXME: this check could probably be removed later */
-    SilcUInt32 my_len = (SilcUInt32) (strchr(file->base, EOF) - file->base);
-    SILC_CONFIG_DEBUG(("file=0x%x name=\"%s\" level=%d line=%lu", (SilcUInt32) file,
-			file->filename, file->level, file->line));
-    if (my_len != file->len) {
-      fprintf(stderr, "FATAL ERROR: saved len and current len does not match!\n");
-      abort();
-    }
     silc_free(file->filename);
     memset(file->base, 'F', file->len);
     silc_free(file->base);
@@ -344,13 +336,9 @@ char *silc_config_read_line(SilcConfigFile *file, SilcUInt32 line)
  found:
   if ((endbuf = strchr(p, '\n'))) {
     len = endbuf - p;
-    ret = silc_calloc(len, sizeof(*ret));
-    strncpy(ret, p, len);
-    ret[len] = '\0';
-  }
-  else {
-    ret = silc_calloc(strlen(p), sizeof(*ret));
-    strcpy(ret, p);
+    ret = silc_memdup(p, len);
+  } else {
+    ret = silc_memdup(p, strlen(p));
   }
   return ret;
 }
@@ -364,11 +352,16 @@ char *silc_config_read_current_line(SilcConfigFile *file)
 
 /* (Private) destroy a SilcConfigEntity */
 
-static void silc_config_destroy(SilcConfigEntity ent)
+static void silc_config_destroy(SilcConfigEntity ent, bool destroy_opts)
 {
   SilcConfigOption *oldopt, *nextopt;
   SILC_CONFIG_DEBUG(("Freeing config entity [ent=0x%x] [opts=0x%x]",
 			(SilcUInt32) ent, (SilcUInt32) ent->opts));
+
+  /* if she wants to preserve options just free the object struct */
+  if (!destroy_opts)
+    goto skip_sect;
+
   for (oldopt = ent->opts; oldopt; oldopt = nextopt) {
     nextopt = oldopt->next;
     memset(oldopt->name, 'F', strlen(oldopt->name) + 1);
@@ -376,6 +369,8 @@ static void silc_config_destroy(SilcConfigEntity ent)
     memset(oldopt, 'F', sizeof(*oldopt));
     silc_free(oldopt);
   }
+
+ skip_sect:
   memset(ent, 'F', sizeof(*ent));
   silc_free(ent);
 }
@@ -642,7 +637,9 @@ int silc_config_main(SilcConfigEntity ent)
    * destroyed. */
  main_cleanup:
   if ((file->level != 0) || (file->included != TRUE))
-    silc_config_destroy(ent);
+    silc_config_destroy(ent, TRUE);
+  else
+    silc_config_destroy(ent, FALSE);
 
  main_end:
   return ret;
