@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: command_reply.c,v 1.91 2002/06/26 18:42:27 priikone Exp $ */
+/* $Id: command_reply.c,v 1.101 2002/10/09 18:59:24 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
@@ -66,7 +66,6 @@ void silc_server_command_reply_process(SilcServer server,
   SilcServerCommandReplyContext ctx;
   SilcCommandPayload payload;
   SilcCommand command;
-  SilcUInt16 ident;
 
   SILC_LOG_DEBUG(("Start"));
 
@@ -85,16 +84,22 @@ void silc_server_command_reply_process(SilcServer server,
   ctx->sock = silc_socket_dup(sock);
   ctx->payload = payload;
   ctx->args = silc_command_get_args(ctx->payload);
-  ident = silc_command_get_ident(ctx->payload);
+  ctx->ident = silc_command_get_ident(ctx->payload);
+  command = silc_command_get(ctx->payload);
+
+  /* Client is not allowed to send reply to all commands */
+  if (sock->type == SILC_SOCKET_TYPE_CLIENT &&
+      command != SILC_COMMAND_WHOIS) {
+    silc_server_command_reply_free(ctx);
+    return;
+  }
       
   /* Check for pending commands and mark to be exeucted */
   ctx->callbacks = 
-    silc_server_command_pending_check(server, ctx, 
-				      silc_command_get(ctx->payload), 
-				      ident, &ctx->callbacks_count);
+    silc_server_command_pending_check(server, command,
+				      ctx->ident, &ctx->callbacks_count);
 
   /* Execute command reply */
-  command = silc_command_get(ctx->payload);
   for (cmd = silc_command_reply_list; cmd->cb; cmd++)
     if (cmd->cmd == command)
       break;
@@ -143,6 +148,7 @@ silc_server_command_process_error(SilcServerCommandReplyContext cmd,
 	if (client) {
 	  silc_server_remove_from_channels(server, NULL, client, TRUE, 
 					   NULL, TRUE);
+	  silc_idlist_del_data(client);
 	  silc_idlist_del_client(server->global_list, client);
 	}
 	silc_free(client_id);
@@ -209,8 +215,7 @@ silc_server_command_reply_whois_save(SilcServerCommandReplyContext cmd)
     client = silc_idlist_add_client(server->global_list, nick, 
 				    strdup(username), 
 				    strdup(realname), client_id, 
-				    cmd->sock->user_data, NULL, 
-				    time(NULL) + 300);
+				    cmd->sock->user_data, NULL, 0);
     if (!client) {
       SILC_LOG_ERROR(("Could not add new client to the ID Cache"));
       return FALSE;
@@ -249,7 +254,7 @@ silc_server_command_reply_whois_save(SilcServerCommandReplyContext cmd)
     /* Create new cache entry */
     silc_idcache_add(global ? server->global_list->clients :
 		     server->local_list->clients, nick, client->id, 
-		     client, 0, &cache); 
+		     client, 0, NULL); 
     silc_free(client_id);
   }
 
@@ -267,16 +272,50 @@ silc_server_command_reply_whois_save(SilcServerCommandReplyContext cmd)
       silc_server_save_user_channels(server, cmd->sock, client, NULL, NULL);
     }
 
-    if (cache)
-      /* If client is global and is not on any channel then add that we'll
-	 expire the entry after a while. */
-      if (global && !silc_hash_table_count(client->channels))
+    /* If client is global and is not on any channel then add that we'll
+       expire the entry after a while. */
+    if (global) {
+      silc_idlist_find_client_by_id(server->global_list, client->id,
+				    FALSE, &cache);
+      if (!silc_hash_table_count(client->channels))
 	cache->expire = time(NULL) + 300;
+      else
+	cache->expire = 0;
+    }
   }
 
   if (fingerprint && flen == sizeof(client->data.fingerprint))
     memcpy(client->data.fingerprint, fingerprint, flen);
 
+  /* Take Requested Attributes if set. */
+  tmp = silc_argument_get_arg_type(cmd->args, 11, &len);
+  if (tmp) {
+    silc_free(client->attrs);
+    client->attrs = silc_memdup(tmp, len);
+  }
+
+  return TRUE;
+}
+
+/* Handle requested attributes reply in WHOIS from client */
+
+static char
+silc_server_command_reply_whois_save_client(SilcServerCommandReplyContext cmd)
+{
+  unsigned char *tmp;
+  SilcUInt32 len;
+  SilcClientEntry client = cmd->sock->user_data;
+
+  /* Take Requested Attributes if set. */
+  tmp = silc_argument_get_arg_type(cmd->args, 11, &len);
+  if (tmp && client) {
+    silc_free(client->attrs);
+    client->attrs = silc_memdup(tmp, len);
+    client->attrs_len = len;
+  }
+
+  client->data.status &= ~SILC_IDLIST_STATUS_RESOLVING;
+
   return TRUE;
 }
 
@@ -293,8 +332,13 @@ SILC_SERVER_CMD_REPLY_FUNC(whois)
 
   COMMAND_CHECK_STATUS;
 
-  if (!silc_server_command_reply_whois_save(cmd))
-    goto out;
+  if (cmd->sock->type != SILC_SOCKET_TYPE_CLIENT) {
+    if (!silc_server_command_reply_whois_save(cmd))
+      goto out;
+  } else {
+    if (!silc_server_command_reply_whois_save_client(cmd))
+      goto out;
+  }
 
   /* Pending callbacks are not executed if this was an list entry */
   if (status != SILC_STATUS_OK &&
@@ -373,6 +417,8 @@ silc_server_command_reply_whowas_save(SilcServerCommandReplyContext cmd)
       return FALSE;
     }
 
+    client->data.status |= SILC_IDLIST_STATUS_RESOLVED;
+    client->data.status &= ~SILC_IDLIST_STATUS_RESOLVING;
     client->data.status &= ~SILC_IDLIST_STATUS_REGISTERED; 
     client->servername = servername;
   } else {
@@ -383,10 +429,13 @@ silc_server_command_reply_whowas_save(SilcServerCommandReplyContext cmd)
 
     silc_free(client->nickname);
     silc_free(client->username);
+    silc_free(client->servername);
     
     client->nickname = nick;
     client->username = strdup(username);
     client->servername = servername;
+    client->data.status |= SILC_IDLIST_STATUS_RESOLVED;
+    client->data.status &= ~SILC_IDLIST_STATUS_RESOLVING;
 
     /* Remove the old cache entry and create a new one */
     silc_idcache_del_by_context(global ? server->global_list->clients :
@@ -396,6 +445,17 @@ silc_server_command_reply_whowas_save(SilcServerCommandReplyContext cmd)
 		     client, 0, NULL);
   }
 
+  /* If client is global and is not on any channel then add that we'll
+     expire the entry after a while. */
+  if (global) {
+    silc_idlist_find_client_by_id(server->global_list, client->id,
+				  FALSE, &cache);
+    if (!silc_hash_table_count(client->channels))
+      cache->expire = SILC_ID_CACHE_EXPIRE_DEF;
+    else
+      cache->expire = 0;
+  }
+
   silc_free(client_id);
 
   return TRUE;
@@ -422,8 +482,13 @@ SILC_SERVER_CMD_REPLY_FUNC(whowas)
   }
 
  out:
+  silc_server_command_process_error(cmd, error);
   SILC_SERVER_PENDING_EXEC(cmd, SILC_COMMAND_WHOWAS);
+  silc_server_command_reply_free(cmd);
+  return;
+
  err:
+  silc_server_command_process_error(cmd, error);
   silc_server_command_reply_free(cmd);
 }
 
@@ -490,8 +555,8 @@ silc_server_command_reply_identify_save(SilcServerCommandReplyContext cmd)
 	 global. */
       client = silc_idlist_add_client(server->global_list, nick, 
 				      info ? strdup(info) : NULL, NULL,
-				      client_id, cmd->sock->user_data, NULL,
-				      time(NULL) + 300);
+				      client_id, cmd->sock->user_data,
+				      NULL, time(NULL) + 300);
       if (!client) {
 	SILC_LOG_ERROR(("Could not add new client to the ID Cache"));
 	goto error;
@@ -525,18 +590,24 @@ silc_server_command_reply_identify_save(SilcServerCommandReplyContext cmd)
       client->data.status &= ~SILC_IDLIST_STATUS_RESOLVING;
       
       if (name) {
-	/* If client is global and is not on any channel then add that we'll
-	   expire the entry after a while. */
-	if (global && !silc_hash_table_count(client->channels) &&
-	    server->server_type == SILC_SERVER)
-	  expire = time(NULL) + 300;
-
 	/* Add new cache entry */
 	silc_idcache_add(global ? server->global_list->clients :
 			 server->local_list->clients, nick, client->id, 
 			 client, expire, NULL);
       }
 
+      /* If client is global and is not on any channel then add that we'll
+         expire the entry after a while. */
+      if (global && server->server_type == SILC_SERVER) {
+	SilcIDCacheEntry cache = NULL;
+        silc_idlist_find_client_by_id(server->global_list, client->id,
+				      FALSE, &cache);
+        if (!silc_hash_table_count(client->channels))
+	  cache->expire = time(NULL) + 300;
+        else
+	  cache->expire = 0;
+      }
+
       silc_free(client_id);
     }
 
